/*
 * bspPIC.c
 *
 *  Created on: 30 Sep 2016
 *      Author: SW_APPLEBY
 */
/*
 ##############################################################
 # (C) 2016 BAE SYSTEMS
 #
 # EmbedWithECOA - "BSP" functions for the PIC
 #
 # $Id: bspPIC.c,v 2.1 2017/11/06 15:33:43 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/

#include <src/bspPIC.h>
#include <stdio.h>
#include <time.h>


static __code unsigned short __at (0x2007) __CONFIG =
#if   PIC==0x16f627
  _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _XT_OSC & _ER_OSC_CLKOUT;
	//
#elif PIC==0x16f873
  _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _DEBUG_OFF & _WRT_ENABLE_ON & _LVP_OFF;
	//
#elif PIC==0x16f876
  _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _DEBUG_OFF & _WRT_ENABLE_ON & _LVP_OFF;
	//
#elif PIC==0x16f876a
  _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _DEBUG_OFF & _LVP_OFF;
#endif

static clock_t clockc = 0;

static short TMRCOUNT = (256+4/*fudge*/)-(100/*usecs)*/);

void pfinit()
{
#if   PIC==0x16f627
	TMR0       = 0;
	OPTION_REG = 0x08;		// PSA = 1 assigns prescalar to WDT, which is then not used...
	CMCON      = 0x07;		// aka ADCON0 // A/D on => RA0:3 digital inputs
	TRISA      = 0xFF;		// PORTA all inputs
	TRISB      = 0x00;		// PORTB all outputs
	//
#elif PIC==0x16f873
	TMR0       = 0;
	OPTION_REG = 0x88;		// PSA = 1 assigns prescalar to WDT, which is then not used...
	ADCON0     = 0x00;		//
	ADCON1     = 0x07;		// RA0:3 digital i/o
	TRISA      = 0x00;		// PORTA all outputs
	TRISB      = 0xFF;		// PORTB all inputs
	//
#elif PIC==0x16f876
	TMR0       = 0;
	OPTION_REG = 0x88;		// PSA = 1 assigns prescalar to WDT, which is then not used...
	ADCON0     = 0x00;		//
	ADCON1     = 0x07;		// RA0:3 digital i/o
	TRISA      = 0x00;		// PORTA all outputs
	TRISB      = 0xFF;		// PORTB all inputs
	//
#elif PIC==0x16f876a
	TMR0       = 0;
	OPTION_REG = 0x88;		// PSA = 1 assigns prescalar to WDT, which is then not used...
	ADCON0     = 0x00;		//
	ADCON1     = 0x07;		// RA0:3 digital i/o
	TRISA      = 0x00;		// PORTA all outputs
	TRISB      = 0xFF;		// PORTB all inputs
#endif
	LEDPORT    = 0x00;		// Turn LEDs off
	T0IE       = 1; 		// enable interrupts
	GIE        = 1;
	TMR1IE     = 1;
	TMR0       = (unsigned char)TMRCOUNT;
}

void Timer0_ISR() __interrupt
{
	if( T0IE && T0IF ){
		TMR0 = TMRCOUNT;
		T0IF = 0; // clear bit
		++clockc;
	}
}

clock_t clock()
{
	return clockc;
}

unsigned short usleep( unsigned short usecs )
	// The timer ticks at 1MHz (Fosc/4), so X ticks == X usecs
{
	unsigned short tcount = ~usecs;//65536 - usecs;
	tcount += 120;
	// Stop TIMER1
	T1CON = 0x00;
	// Load the counter
	TMR1H = tcount >> 8;
	TMR1L = tcount & 0xFF;
	// Clear the overflow flag so that it can get set again...
	TMR1IF = 0;
	// Start TIMER1
	T1CON = 0x01;
	// Wait for overflow
	while( !TMR1IF );
	return 0;
}

unsigned msleep( unsigned msecs )
{
	while( msecs > 0 ){
		usleep( 1000 );
		--msecs;
	}
	return 0;
}

unsigned sleep( unsigned secs )
{
	while( secs > 0 ){
		msleep( 1000 );
		--secs;
	}
	return 0;
}

