/*
 * EwECOA_platform.c
 *
 *  Created on: 28 Sep 2016
 *      Author: SW_APPLEBY
 */
/*
 ##############################################################
 # (C) 2016 BAE SYSTEMS
 #
 # EmbedWithECOA - Bare minimum ECOA S/W Platform implementation
 #
 # $Id: EwECOA_platform.c,v 2.6 2017/11/06 15:33:43 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/

#include <ECOA.h>
#include "bspPIC.H"
#include "EwECOA_platform.h"
#include <EwECOA_modMain_Im.h>
#include <EwECOAIF_modMain_Im.h>

#if defined(__CYGWIN__)
#include <unistd.h>
#endif

EwECOA_modMain_Im__context          EwECOA_modMain_Inst_context;
static EwECOA_modMain_Im_PlatformContext   EwECOA_modMain_Inst_pContext;

EwECOAIF_modMain_Im__context        EwECOAIF_modMain_Inst_context;
static EwECOAIF_modMain_Im_PlatformContext EwECOAIF_modMain_Inst_pContext;

#define EwECOA_Ticker_period 100 /* msecs */ * CLOCKS_PER_SEC / 1000

ECOA__duration ClockResolution = {0,1000000000L/CLOCKS_PER_SEC};

extern void pollButtons();

void EwECOA_platform_log( char *msg, int msgLen, LogLevel level )
{
	/* maybe send to serial port ? */
}

void hrClock( ECOA__hr_time *hr_time )
{
	clock_t now = clock();
	if( hr_time ){
		hr_time->seconds = now/CLOCKS_PER_SEC;
		hr_time->nanoseconds = now * (1000000000L/CLOCKS_PER_SEC) % 1000000000L;
	}
}

void gblClock( ECOA__global_time *gbl_time )
{
	hrClock( (ECOA__hr_time*)gbl_time );
}

//###########################################################################################
extern unsigned msleep( unsigned msecs );

int EwECOA_platform__main()
{
	clock_t nextTime = clock() + EwECOA_Ticker_period;
	//
	pfinit();
	//
	// Initialize the Trigger & Module states
	EwECOA_modMain_Inst_pContext.tickerState  = ECOA_Platform__module_states_type_IDLE;
	EwECOA_modMain_Inst_pContext.moduleState  = ECOA_Platform__module_states_type_IDLE;
	EwECOAIF_modMain_Inst_pContext.moduleState  = ECOA_Platform__module_states_type_IDLE;
	//
	// INITIALIZE the Triggers & Modules themselves
	EwECOA_modMain_Inst_pContext.tickerState  = ECOA_Platform__module_states_type_READY;
	EwECOA_modMain_Im__INITIALIZE__received( &EwECOA_modMain_Inst_context );
	EwECOA_modMain_Inst_pContext.moduleState = ECOA_Platform__module_states_type_READY;
	EwECOAIF_modMain_Im__INITIALIZE__received( &EwECOAIF_modMain_Inst_context );
	EwECOAIF_modMain_Inst_pContext.moduleState = ECOA_Platform__module_states_type_READY;
	//
	// START the Triggers & Modules
	EwECOA_modMain_Inst_pContext.tickerState  = ECOA_Platform__module_states_type_RUNNING;
	EwECOA_modMain_Im__START__received( &EwECOA_modMain_Inst_context );
	EwECOA_modMain_Inst_pContext.moduleState = ECOA_Platform__module_states_type_RUNNING;
	EwECOAIF_modMain_Im__START__received( &EwECOAIF_modMain_Inst_context );
	EwECOAIF_modMain_Inst_pContext.moduleState = ECOA_Platform__module_states_type_RUNNING;
	//
	for(;;){
		if( EwECOA_modMain_Inst_pContext.tickerState == ECOA_Platform__module_states_type_RUNNING ){
			EwECOA_modMain_Im__Tick__received( &EwECOA_modMain_Inst_context );
		}
		pollButtons();
		msleep( 100/*msecs*/ );   // TODO this needs fixing...
//		msleep( nextTime - clock() );
//		nextTime += EwECOA_Ticker_period;
	}
}

//###########################################################################################

void EwECOA_platform_DisplayCode_send( const ECOA__uint8 code )
{
	EwECOAIF_modMain_Im__DisplayCode__received( &EwECOAIF_modMain_Inst_context, code );
}

//###########################################################################################

void EwECOA_platform_PatternASelected_send()
{
	EwECOA_modMain_Im__PatternASelected__received( &EwECOA_modMain_Inst_context );
}

void EwECOA_platform_PatternBSelected_send()
{
	EwECOA_modMain_Im__PatternBSelected__received( &EwECOA_modMain_Inst_context );
}

void EwECOA_platform_PatternCSelected_send()
{
	EwECOA_modMain_Im__PatternCSelected__received( &EwECOA_modMain_Inst_context );
}

void EwECOA_platform_PatternDSelected_send()
{
	EwECOA_modMain_Im__PatternDSelected__received( &EwECOA_modMain_Inst_context );
}

